"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import tendermint.crypto.proof_pb2
import tendermint.types.validator_pb2
import tendermint.version.types_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _BlockIDFlag:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _BlockIDFlagEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_BlockIDFlag.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    BLOCK_ID_FLAG_UNKNOWN: _BlockIDFlag.ValueType  # 0
    BLOCK_ID_FLAG_ABSENT: _BlockIDFlag.ValueType  # 1
    BLOCK_ID_FLAG_COMMIT: _BlockIDFlag.ValueType  # 2
    BLOCK_ID_FLAG_NIL: _BlockIDFlag.ValueType  # 3
class BlockIDFlag(_BlockIDFlag, metaclass=_BlockIDFlagEnumTypeWrapper):
    """BlockIdFlag indicates which BlcokID the signature is for"""
    pass

BLOCK_ID_FLAG_UNKNOWN: BlockIDFlag.ValueType  # 0
BLOCK_ID_FLAG_ABSENT: BlockIDFlag.ValueType  # 1
BLOCK_ID_FLAG_COMMIT: BlockIDFlag.ValueType  # 2
BLOCK_ID_FLAG_NIL: BlockIDFlag.ValueType  # 3
global___BlockIDFlag = BlockIDFlag


class _SignedMsgType:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _SignedMsgTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SignedMsgType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SIGNED_MSG_TYPE_UNKNOWN: _SignedMsgType.ValueType  # 0
    SIGNED_MSG_TYPE_PREVOTE: _SignedMsgType.ValueType  # 1
    """Votes"""

    SIGNED_MSG_TYPE_PRECOMMIT: _SignedMsgType.ValueType  # 2
    SIGNED_MSG_TYPE_PROPOSAL: _SignedMsgType.ValueType  # 32
    """Proposals"""

class SignedMsgType(_SignedMsgType, metaclass=_SignedMsgTypeEnumTypeWrapper):
    """SignedMsgType is a type of signed message in the consensus."""
    pass

SIGNED_MSG_TYPE_UNKNOWN: SignedMsgType.ValueType  # 0
SIGNED_MSG_TYPE_PREVOTE: SignedMsgType.ValueType  # 1
"""Votes"""

SIGNED_MSG_TYPE_PRECOMMIT: SignedMsgType.ValueType  # 2
SIGNED_MSG_TYPE_PROPOSAL: SignedMsgType.ValueType  # 32
"""Proposals"""

global___SignedMsgType = SignedMsgType


class PartSetHeader(google.protobuf.message.Message):
    """PartsetHeader"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOTAL_FIELD_NUMBER: builtins.int
    HASH_FIELD_NUMBER: builtins.int
    total: builtins.int
    hash: builtins.bytes
    def __init__(self,
        *,
        total: builtins.int = ...,
        hash: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["hash",b"hash","total",b"total"]) -> None: ...
global___PartSetHeader = PartSetHeader

class Part(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INDEX_FIELD_NUMBER: builtins.int
    BYTES_FIELD_NUMBER: builtins.int
    PROOF_FIELD_NUMBER: builtins.int
    index: builtins.int
    bytes: builtins.bytes
    @property
    def proof(self) -> tendermint.crypto.proof_pb2.Proof: ...
    def __init__(self,
        *,
        index: builtins.int = ...,
        bytes: builtins.bytes = ...,
        proof: typing.Optional[tendermint.crypto.proof_pb2.Proof] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["proof",b"proof"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bytes",b"bytes","index",b"index","proof",b"proof"]) -> None: ...
global___Part = Part

class BlockID(google.protobuf.message.Message):
    """BlockID"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HASH_FIELD_NUMBER: builtins.int
    PART_SET_HEADER_FIELD_NUMBER: builtins.int
    hash: builtins.bytes
    @property
    def part_set_header(self) -> global___PartSetHeader: ...
    def __init__(self,
        *,
        hash: builtins.bytes = ...,
        part_set_header: typing.Optional[global___PartSetHeader] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["part_set_header",b"part_set_header"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["hash",b"hash","part_set_header",b"part_set_header"]) -> None: ...
global___BlockID = BlockID

class Header(google.protobuf.message.Message):
    """--------------------------------

    Header defines the structure of a Tendermint block header.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VERSION_FIELD_NUMBER: builtins.int
    CHAIN_ID_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    TIME_FIELD_NUMBER: builtins.int
    LAST_BLOCK_ID_FIELD_NUMBER: builtins.int
    LAST_COMMIT_HASH_FIELD_NUMBER: builtins.int
    DATA_HASH_FIELD_NUMBER: builtins.int
    VALIDATORS_HASH_FIELD_NUMBER: builtins.int
    NEXT_VALIDATORS_HASH_FIELD_NUMBER: builtins.int
    CONSENSUS_HASH_FIELD_NUMBER: builtins.int
    APP_HASH_FIELD_NUMBER: builtins.int
    LAST_RESULTS_HASH_FIELD_NUMBER: builtins.int
    EVIDENCE_HASH_FIELD_NUMBER: builtins.int
    PROPOSER_ADDRESS_FIELD_NUMBER: builtins.int
    @property
    def version(self) -> tendermint.version.types_pb2.Consensus:
        """basic block info"""
        pass
    chain_id: typing.Text
    height: builtins.int
    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def last_block_id(self) -> global___BlockID:
        """prev block info"""
        pass
    last_commit_hash: builtins.bytes
    """hashes of block data
    commit from validators from the last block
    """

    data_hash: builtins.bytes
    """transactions"""

    validators_hash: builtins.bytes
    """hashes from the app output from the prev block
    validators for the current block
    """

    next_validators_hash: builtins.bytes
    """validators for the next block"""

    consensus_hash: builtins.bytes
    """consensus params for current block"""

    app_hash: builtins.bytes
    """state after txs from the previous block"""

    last_results_hash: builtins.bytes
    """root hash of all results from the txs from the previous block"""

    evidence_hash: builtins.bytes
    """consensus info
    evidence included in the block
    """

    proposer_address: builtins.bytes
    """original proposer of the block"""

    def __init__(self,
        *,
        version: typing.Optional[tendermint.version.types_pb2.Consensus] = ...,
        chain_id: typing.Text = ...,
        height: builtins.int = ...,
        time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        last_block_id: typing.Optional[global___BlockID] = ...,
        last_commit_hash: builtins.bytes = ...,
        data_hash: builtins.bytes = ...,
        validators_hash: builtins.bytes = ...,
        next_validators_hash: builtins.bytes = ...,
        consensus_hash: builtins.bytes = ...,
        app_hash: builtins.bytes = ...,
        last_results_hash: builtins.bytes = ...,
        evidence_hash: builtins.bytes = ...,
        proposer_address: builtins.bytes = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["last_block_id",b"last_block_id","time",b"time","version",b"version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_hash",b"app_hash","chain_id",b"chain_id","consensus_hash",b"consensus_hash","data_hash",b"data_hash","evidence_hash",b"evidence_hash","height",b"height","last_block_id",b"last_block_id","last_commit_hash",b"last_commit_hash","last_results_hash",b"last_results_hash","next_validators_hash",b"next_validators_hash","proposer_address",b"proposer_address","time",b"time","validators_hash",b"validators_hash","version",b"version"]) -> None: ...
global___Header = Header

class Data(google.protobuf.message.Message):
    """Data contains the set of transactions included in the block"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TXS_FIELD_NUMBER: builtins.int
    @property
    def txs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]:
        """Txs that will be applied by state @ block.Height+1.
        NOTE: not all txs here are valid.  We're just agreeing on the order first.
        This means that block.AppHash does not include these txs.
        """
        pass
    def __init__(self,
        *,
        txs: typing.Optional[typing.Iterable[builtins.bytes]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["txs",b"txs"]) -> None: ...
global___Data = Data

class Vote(google.protobuf.message.Message):
    """Vote represents a prevote, precommit, or commit vote from validators for
    consensus.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TYPE_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    ROUND_FIELD_NUMBER: builtins.int
    BLOCK_ID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_INDEX_FIELD_NUMBER: builtins.int
    SIGNATURE_FIELD_NUMBER: builtins.int
    type: global___SignedMsgType.ValueType
    height: builtins.int
    round: builtins.int
    @property
    def block_id(self) -> global___BlockID:
        """zero if vote is nil."""
        pass
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    validator_address: builtins.bytes
    validator_index: builtins.int
    signature: builtins.bytes
    def __init__(self,
        *,
        type: global___SignedMsgType.ValueType = ...,
        height: builtins.int = ...,
        round: builtins.int = ...,
        block_id: typing.Optional[global___BlockID] = ...,
        timestamp: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        validator_address: builtins.bytes = ...,
        validator_index: builtins.int = ...,
        signature: builtins.bytes = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["block_id",b"block_id","timestamp",b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_id",b"block_id","height",b"height","round",b"round","signature",b"signature","timestamp",b"timestamp","type",b"type","validator_address",b"validator_address","validator_index",b"validator_index"]) -> None: ...
global___Vote = Vote

class Commit(google.protobuf.message.Message):
    """Commit contains the evidence that a block was committed by a set of validators."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HEIGHT_FIELD_NUMBER: builtins.int
    ROUND_FIELD_NUMBER: builtins.int
    BLOCK_ID_FIELD_NUMBER: builtins.int
    SIGNATURES_FIELD_NUMBER: builtins.int
    height: builtins.int
    round: builtins.int
    @property
    def block_id(self) -> global___BlockID: ...
    @property
    def signatures(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CommitSig]: ...
    def __init__(self,
        *,
        height: builtins.int = ...,
        round: builtins.int = ...,
        block_id: typing.Optional[global___BlockID] = ...,
        signatures: typing.Optional[typing.Iterable[global___CommitSig]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["block_id",b"block_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_id",b"block_id","height",b"height","round",b"round","signatures",b"signatures"]) -> None: ...
global___Commit = Commit

class CommitSig(google.protobuf.message.Message):
    """CommitSig is a part of the Vote included in a Commit."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BLOCK_ID_FLAG_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    SIGNATURE_FIELD_NUMBER: builtins.int
    block_id_flag: global___BlockIDFlag.ValueType
    validator_address: builtins.bytes
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    signature: builtins.bytes
    def __init__(self,
        *,
        block_id_flag: global___BlockIDFlag.ValueType = ...,
        validator_address: builtins.bytes = ...,
        timestamp: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        signature: builtins.bytes = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["timestamp",b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_id_flag",b"block_id_flag","signature",b"signature","timestamp",b"timestamp","validator_address",b"validator_address"]) -> None: ...
global___CommitSig = CommitSig

class Proposal(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TYPE_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    ROUND_FIELD_NUMBER: builtins.int
    POL_ROUND_FIELD_NUMBER: builtins.int
    BLOCK_ID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    SIGNATURE_FIELD_NUMBER: builtins.int
    type: global___SignedMsgType.ValueType
    height: builtins.int
    round: builtins.int
    pol_round: builtins.int
    @property
    def block_id(self) -> global___BlockID: ...
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    signature: builtins.bytes
    def __init__(self,
        *,
        type: global___SignedMsgType.ValueType = ...,
        height: builtins.int = ...,
        round: builtins.int = ...,
        pol_round: builtins.int = ...,
        block_id: typing.Optional[global___BlockID] = ...,
        timestamp: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        signature: builtins.bytes = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["block_id",b"block_id","timestamp",b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_id",b"block_id","height",b"height","pol_round",b"pol_round","round",b"round","signature",b"signature","timestamp",b"timestamp","type",b"type"]) -> None: ...
global___Proposal = Proposal

class SignedHeader(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HEADER_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def header(self) -> global___Header: ...
    @property
    def commit(self) -> global___Commit: ...
    def __init__(self,
        *,
        header: typing.Optional[global___Header] = ...,
        commit: typing.Optional[global___Commit] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["commit",b"commit","header",b"header"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit",b"commit","header",b"header"]) -> None: ...
global___SignedHeader = SignedHeader

class LightBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SIGNED_HEADER_FIELD_NUMBER: builtins.int
    VALIDATOR_SET_FIELD_NUMBER: builtins.int
    @property
    def signed_header(self) -> global___SignedHeader: ...
    @property
    def validator_set(self) -> tendermint.types.validator_pb2.ValidatorSet: ...
    def __init__(self,
        *,
        signed_header: typing.Optional[global___SignedHeader] = ...,
        validator_set: typing.Optional[tendermint.types.validator_pb2.ValidatorSet] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["signed_header",b"signed_header","validator_set",b"validator_set"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["signed_header",b"signed_header","validator_set",b"validator_set"]) -> None: ...
global___LightBlock = LightBlock

class BlockMeta(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BLOCK_ID_FIELD_NUMBER: builtins.int
    BLOCK_SIZE_FIELD_NUMBER: builtins.int
    HEADER_FIELD_NUMBER: builtins.int
    NUM_TXS_FIELD_NUMBER: builtins.int
    @property
    def block_id(self) -> global___BlockID: ...
    block_size: builtins.int
    @property
    def header(self) -> global___Header: ...
    num_txs: builtins.int
    def __init__(self,
        *,
        block_id: typing.Optional[global___BlockID] = ...,
        block_size: builtins.int = ...,
        header: typing.Optional[global___Header] = ...,
        num_txs: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["block_id",b"block_id","header",b"header"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_id",b"block_id","block_size",b"block_size","header",b"header","num_txs",b"num_txs"]) -> None: ...
global___BlockMeta = BlockMeta

class TxProof(google.protobuf.message.Message):
    """TxProof represents a Merkle proof of the presence of a transaction in the Merkle tree."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ROOT_HASH_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    PROOF_FIELD_NUMBER: builtins.int
    root_hash: builtins.bytes
    data: builtins.bytes
    @property
    def proof(self) -> tendermint.crypto.proof_pb2.Proof: ...
    def __init__(self,
        *,
        root_hash: builtins.bytes = ...,
        data: builtins.bytes = ...,
        proof: typing.Optional[tendermint.crypto.proof_pb2.Proof] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["proof",b"proof"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data",b"data","proof",b"proof","root_hash",b"root_hash"]) -> None: ...
global___TxProof = TxProof



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Snapshot(google.protobuf.message.Message):
    """Snapshot contains Tendermint state sync snapshot info."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HEIGHT_FIELD_NUMBER: builtins.int
    FORMAT_FIELD_NUMBER: builtins.int
    CHUNKS_FIELD_NUMBER: builtins.int
    HASH_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    height: builtins.int
    format: builtins.int
    chunks: builtins.int
    hash: builtins.bytes
    @property
    def metadata(self) -> global___Metadata: ...
    def __init__(self,
        *,
        height: builtins.int = ...,
        format: builtins.int = ...,
        chunks: builtins.int = ...,
        hash: builtins.bytes = ...,
        metadata: typing.Optional[global___Metadata] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["metadata",b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunks",b"chunks","format",b"format","hash",b"hash","height",b"height","metadata",b"metadata"]) -> None: ...
global___Snapshot = Snapshot

class Metadata(google.protobuf.message.Message):
    """Metadata contains SDK-specific snapshot metadata."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CHUNK_HASHES_FIELD_NUMBER: builtins.int
    @property
    def chunk_hashes(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]:
        """SHA-256 chunk hashes"""
        pass
    def __init__(self,
        *,
        chunk_hashes: typing.Optional[typing.Iterable[builtins.bytes]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk_hashes",b"chunk_hashes"]) -> None: ...
global___Metadata = Metadata

class SnapshotItem(google.protobuf.message.Message):
    """SnapshotItem is an item contained in a rootmulti.Store snapshot."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    STORE_FIELD_NUMBER: builtins.int
    IAVL_FIELD_NUMBER: builtins.int
    EXTENSION_FIELD_NUMBER: builtins.int
    EXTENSION_PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def store(self) -> global___SnapshotStoreItem: ...
    @property
    def iavl(self) -> global___SnapshotIAVLItem: ...
    @property
    def extension(self) -> global___SnapshotExtensionMeta: ...
    @property
    def extension_payload(self) -> global___SnapshotExtensionPayload: ...
    def __init__(self,
        *,
        store: typing.Optional[global___SnapshotStoreItem] = ...,
        iavl: typing.Optional[global___SnapshotIAVLItem] = ...,
        extension: typing.Optional[global___SnapshotExtensionMeta] = ...,
        extension_payload: typing.Optional[global___SnapshotExtensionPayload] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["extension",b"extension","extension_payload",b"extension_payload","iavl",b"iavl","item",b"item","store",b"store"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["extension",b"extension","extension_payload",b"extension_payload","iavl",b"iavl","item",b"item","store",b"store"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["item",b"item"]) -> typing.Optional[typing_extensions.Literal["store","iavl","extension","extension_payload"]]: ...
global___SnapshotItem = SnapshotItem

class SnapshotStoreItem(google.protobuf.message.Message):
    """SnapshotStoreItem contains metadata about a snapshotted store."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NAME_FIELD_NUMBER: builtins.int
    name: typing.Text
    def __init__(self,
        *,
        name: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name",b"name"]) -> None: ...
global___SnapshotStoreItem = SnapshotStoreItem

class SnapshotIAVLItem(google.protobuf.message.Message):
    """SnapshotIAVLItem is an exported IAVL node."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    key: builtins.bytes
    value: builtins.bytes
    version: builtins.int
    """version is block height"""

    height: builtins.int
    """height is depth of the tree."""

    def __init__(self,
        *,
        key: builtins.bytes = ...,
        value: builtins.bytes = ...,
        version: builtins.int = ...,
        height: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["height",b"height","key",b"key","value",b"value","version",b"version"]) -> None: ...
global___SnapshotIAVLItem = SnapshotIAVLItem

class SnapshotExtensionMeta(google.protobuf.message.Message):
    """SnapshotExtensionMeta contains metadata about an external snapshotter."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NAME_FIELD_NUMBER: builtins.int
    FORMAT_FIELD_NUMBER: builtins.int
    name: typing.Text
    format: builtins.int
    def __init__(self,
        *,
        name: typing.Text = ...,
        format: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["format",b"format","name",b"name"]) -> None: ...
global___SnapshotExtensionMeta = SnapshotExtensionMeta

class SnapshotExtensionPayload(google.protobuf.message.Message):
    """SnapshotExtensionPayload contains payloads of an external snapshotter."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAYLOAD_FIELD_NUMBER: builtins.int
    payload: builtins.bytes
    def __init__(self,
        *,
        payload: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["payload",b"payload"]) -> None: ...
global___SnapshotExtensionPayload = SnapshotExtensionPayload



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Params(google.protobuf.message.Message):
    """Params defines the parameters for the module."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    STARTINGPOOLNUMBER_FIELD_NUMBER: builtins.int
    POOL_CREATION_FEE_FIELD_NUMBER: builtins.int
    WHITELISTED_ASSET_FIELD_NUMBER: builtins.int
    startingPoolNumber: builtins.int
    """The start pool number, i.e. the first pool number that isn't taken yet."""

    @property
    def pool_creation_fee(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """The cost of creating a pool, taken from the pool creator's account."""
        pass
    @property
    def whitelisted_asset(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
        """The assets that can be used to create liquidity pools"""
        pass
    def __init__(self,
        *,
        startingPoolNumber: builtins.int = ...,
        pool_creation_fee: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        whitelisted_asset: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_creation_fee",b"pool_creation_fee","startingPoolNumber",b"startingPoolNumber","whitelisted_asset",b"whitelisted_asset"]) -> None: ...
global___Params = Params



# # Copyright 2022 Nibiru Labs
# #
# # Licensed under the Apache License, Version 2.0 (the "License");
# # you may not use this file except in compliance with the License.
# # You may obtain a copy of the License at
# #
# #     http://www.apache.org/licenses/LICENSE-2.0
# #
# # Unless required by applicable law or agreed to in writing, software
# # distributed under the License is distributed on an "AS IS" BASIS,
# # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# # See the License for the specific language governing permissions and
# # limitations under the License.

# import asyncio
# import logging

# from nibiru.composer import Composer as ProtoMsgComposer
# from nibiru.client import Client
# from nibiru.transaction import Transaction
# from nibiru.network import Network
# from nibiru.wallet import PrivateKey


# async def main() -> None:
#     # select network: local, testnet, mainnet
#     network = Network.local()
#     composer = ProtoMsgComposer(network=network.string())

#     # initialize grpc client
#     client = Client(network, insecure=True)
#     await client.sync_timeout_height()

#     # load account
#     priv_key = PrivateKey.from_mnemonic("guard cream sadness conduct invite crumble clock pudding hole grit liar hotel maid produce squeeze return argue turtle know drive eight casino maze host")
#     pub_key = priv_key.to_public_key()
#     address = await pub_key.to_address().async_init_num_seq(network.lcd_endpoint)
#     subaccount_id = address.get_subaccount_id(index=0)

#     # prepare tx msg
#     msg = composer.msg_revoke(
#         granter = "inj14au322k9munkmx5wrchz9q30juf5wjgz2cfqku",
#         grantee = "inj1hkhdaj2a2clmq5jq6mspsggqs32vynpk228q3r",
#         msg_type = "/nibiru.exchange.v1beta1.MsgCreateSpotLimitOrder"
#     )

#     # build sim tx
#     tx = (
#         Transaction()
#         .with_messages(msg)
#         .with_sequence(address.get_sequence())
#         .with_account_num(address.get_number())
#         .with_chain_id(network.chain_id)
#         .with_signer(priv_key)
#     )
#     sim_tx_raw_bytes = tx.get_signed_tx_data()

#     # simulate tx
#     (sim_res, success) = await client.simulate_tx(sim_tx_raw_bytes)
#     if not success:
#         print(sim_res)
#         return

#     # build tx
#     gas_price = 500000000
#     gas_limit = sim_res.gas_info.gas_used + 20000 # add 20k for gas, fee computation
#     gas_fee = '{:.18f}'.format((gas_price * gas_limit) / pow(10, 18)).rstrip('0')
#     fee = [composer.Coin(
#         amount=gas_price * gas_limit,
#         denom=network.fee_denom,
#     )]
#     tx = tx.with_gas(gas_limit).with_fee(fee).with_memo('').with_timeout_height(client.timeout_height)
#     tx_raw_bytes = tx.get_signed_tx_data()

#     # broadcast tx: send_tx_async_mode, send_tx_sync_mode, send_tx_block_mode
#     res = await client.send_tx_sync_mode(tx_raw_bytes)
#     print(res)
#     print("gas wanted: {}".format(gas_limit))
#     print("gas fee: {} unibi".format(gas_fee))

# if __name__ == "__main__":
#     logging.basicConfig(level=logging.INFO)
#     asyncio.get_event_loop().run_until_complete(main())



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.base.reflection.v2alpha1.reflection_pb2
import grpc

class ReflectionServiceStub:
    """ReflectionService defines a service for application reflection."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    GetAuthnDescriptor: grpc.UnaryUnaryMultiCallable[
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetAuthnDescriptorRequest,
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetAuthnDescriptorResponse]
    """GetAuthnDescriptor returns information on how to authenticate transactions in the application
    NOTE: this RPC is still experimental and might be subject to breaking changes or removal in
    future releases of the cosmos-sdk.
    """

    GetChainDescriptor: grpc.UnaryUnaryMultiCallable[
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetChainDescriptorRequest,
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetChainDescriptorResponse]
    """GetChainDescriptor returns the description of the chain"""

    GetCodecDescriptor: grpc.UnaryUnaryMultiCallable[
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetCodecDescriptorRequest,
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetCodecDescriptorResponse]
    """GetCodecDescriptor returns the descriptor of the codec of the application"""

    GetConfigurationDescriptor: grpc.UnaryUnaryMultiCallable[
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetConfigurationDescriptorRequest,
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetConfigurationDescriptorResponse]
    """GetConfigurationDescriptor returns the descriptor for the sdk.Config of the application"""

    GetQueryServicesDescriptor: grpc.UnaryUnaryMultiCallable[
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetQueryServicesDescriptorRequest,
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetQueryServicesDescriptorResponse]
    """GetQueryServicesDescriptor returns the available gRPC queryable services of the application"""

    GetTxDescriptor: grpc.UnaryUnaryMultiCallable[
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetTxDescriptorRequest,
        cosmos.base.reflection.v2alpha1.reflection_pb2.GetTxDescriptorResponse]
    """GetTxDescriptor returns information on the used transaction object and available msgs that can be used"""


class ReflectionServiceServicer(metaclass=abc.ABCMeta):
    """ReflectionService defines a service for application reflection."""
    @abc.abstractmethod
    def GetAuthnDescriptor(self,
        request: cosmos.base.reflection.v2alpha1.reflection_pb2.GetAuthnDescriptorRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.base.reflection.v2alpha1.reflection_pb2.GetAuthnDescriptorResponse:
        """GetAuthnDescriptor returns information on how to authenticate transactions in the application
        NOTE: this RPC is still experimental and might be subject to breaking changes or removal in
        future releases of the cosmos-sdk.
        """
        pass

    @abc.abstractmethod
    def GetChainDescriptor(self,
        request: cosmos.base.reflection.v2alpha1.reflection_pb2.GetChainDescriptorRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.base.reflection.v2alpha1.reflection_pb2.GetChainDescriptorResponse:
        """GetChainDescriptor returns the description of the chain"""
        pass

    @abc.abstractmethod
    def GetCodecDescriptor(self,
        request: cosmos.base.reflection.v2alpha1.reflection_pb2.GetCodecDescriptorRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.base.reflection.v2alpha1.reflection_pb2.GetCodecDescriptorResponse:
        """GetCodecDescriptor returns the descriptor of the codec of the application"""
        pass

    @abc.abstractmethod
    def GetConfigurationDescriptor(self,
        request: cosmos.base.reflection.v2alpha1.reflection_pb2.GetConfigurationDescriptorRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.base.reflection.v2alpha1.reflection_pb2.GetConfigurationDescriptorResponse:
        """GetConfigurationDescriptor returns the descriptor for the sdk.Config of the application"""
        pass

    @abc.abstractmethod
    def GetQueryServicesDescriptor(self,
        request: cosmos.base.reflection.v2alpha1.reflection_pb2.GetQueryServicesDescriptorRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.base.reflection.v2alpha1.reflection_pb2.GetQueryServicesDescriptorResponse:
        """GetQueryServicesDescriptor returns the available gRPC queryable services of the application"""
        pass

    @abc.abstractmethod
    def GetTxDescriptor(self,
        request: cosmos.base.reflection.v2alpha1.reflection_pb2.GetTxDescriptorRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.base.reflection.v2alpha1.reflection_pb2.GetTxDescriptorResponse:
        """GetTxDescriptor returns information on the used transaction object and available msgs that can be used"""
        pass


def add_ReflectionServiceServicer_to_server(servicer: ReflectionServiceServicer, server: grpc.Server) -> None: ...



import pytest
import aiohttp
import json
import ecdsa
import sha3
import grpc
import base64

from typing import Any, Dict, List
from nibiru.chain_client._wallet import (
    generate_wallet,
    privkey_to_address,
    privkey_to_pubkey,
    pubkey_to_address,
    seed_to_privkey,
    DEFAULT_BECH32_HRP,
)

from nibiru.chain_client._typings import SyncMode
import nibiru.exchange_api.nibiru_accounts_rpc_pb2 as accounts_rpc_pb
import nibiru.exchange_api.nibiru_accounts_rpc_pb2_grpc as accounts_rpc_grpc

MIN_GAS_PRICE = 500000000

class Transaction:

    def __init__(
        self,
        *,
        privkey: bytes,
        account_num: int,
        sequence: int,
        fee: int,
        gas: int,
        fee_denom: str = "inj",
        memo: str = "",
        chain_id: str = "nibiru-888",
        hrp: str = DEFAULT_BECH32_HRP,
        sync_mode: SyncMode = "block",
    ) -> None:
        self._privkey = privkey
        self._account_num = account_num
        self._sequence = sequence
        self._fee = fee
        self._fee_denom = fee_denom
        self._gas = gas
        self._memo = memo
        self._chain_id = chain_id
        self._hrp = hrp
        self._sync_mode = sync_mode
        self._msgs: List[dict] = []


    def add_cosmos_bank_msg_send(self, recipient: str, amount: int, denom: str = "inj") -> None:
        msg = {
            "type": "cosmos-sdk/MsgSend",
            "value": {
                "from_address": privkey_to_address(self._privkey, hrp=self._hrp),
                "to_address": recipient,
                "amount": [{"denom": denom, "amount": str(amount)}],
            },
        }
        self._msgs.append(msg)

    # Nibiru Exchange Module

    def add_exchange_msg_deposit(self, subaccount: str, amount: int, denom: str = "inj") -> None:
        msg = {
            "type": "exchange/MsgDeposit",
            "value": {
                "sender": privkey_to_address(self._privkey, hrp=self._hrp),
                "subaccount_id": subaccount,
                "amount": {"denom": denom, "amount": str(amount)},
            },
        }
        self._msgs.append(msg)

    def get_signed(self) -> str:
        pubkey = privkey_to_pubkey(self._privkey)
        base64_pubkey = base64.b64encode(pubkey).decode("utf-8")
        signed_tx = {
            "tx": {
                "msg": self._msgs,
                "fee": {
                    "gas": str(self._gas),
                    "amount": [{"denom": self._fee_denom, "amount": str(self._fee)}],
                },
                "memo": self._memo,
                "signatures": [
                    {
                        "signature": self._sign(),
                        "pub_key": {"type": "nibiru/PubKeyEthSecp256k1", "value": base64_pubkey},
                        "account_number": str(self._account_num),
                        "sequence": str(self._sequence),
                    }
                ],
            },
            "mode": self._sync_mode,
        }
        return json.dumps(signed_tx, separators=(",", ":"))

    def _sign(self) -> str:
        message_str = json.dumps(
            self._get_sign_message(), separators=(",", ":"), sort_keys=True)
        message_bytes = message_str.encode("utf-8")

        privkey = ecdsa.SigningKey.from_string(
            self._privkey, curve=ecdsa.SECP256k1)
        signature_compact_keccak = privkey.sign_deterministic(
            message_bytes, hashfunc=sha3.keccak_256, sigencode=ecdsa.util.sigencode_string_canonize
        )
        signature_base64_str = base64.b64encode(
            signature_compact_keccak).decode("utf-8")
        return signature_base64_str

    def _get_sign_message(self) -> Dict[str, Any]:
        return {
            "chain_id": self._chain_id,
            "account_number": str(self._account_num),
            "fee": {
                "gas": str(self._gas),
                "amount": [{"amount": str(self._fee), "denom": self._fee_denom}],
            },
            "memo": self._memo,
            "sequence": str(self._sequence),
            "msgs": self._msgs,
        }


async def get_account_num_seq(address: str) -> (int, int):
    async with aiohttp.ClientSession() as session:
        async with session.request(
            'GET', 'http://staking-lcd-testnet.nibiru.network/cosmos/auth/v1beta1/accounts/' + address,
            headers={'Accept-Encoding': 'application/json'},
        ) as response:
            if response.status != 200:
                print(await response.text())
                raise ValueError("HTTP response status", response.status)

            resp = json.loads(await response.text())
            acc = resp['account']['base_account']
            return acc['account_number'], acc['sequence']

async def post_tx(tx_json: str):
    async with aiohttp.ClientSession() as session:
        async with session.request(
            'POST', 'http://staking-lcd-testnet.nibiru.network/txs', data=tx_json,
            headers={'Content-Type': 'application/json'},
        ) as response:
            if response.status != 200:
                print(await response.text())
                raise ValueError("HTTP response status", response.status)

            resp = json.loads(await response.text())
            if 'code' in resp:
                print("Response:", resp)
                raise ValueError('sdk error %d: %s' % (resp['code'], resp['raw_log']))

            return resp['txhash']

@pytest.fixture
async def msg_send():
    sender_pk = seed_to_privkey(
        "physical page glare junk return scale subject river token door mirror title"
    )
    sender_acc_addr = privkey_to_address(sender_pk)
    print("Sender Account:", sender_acc_addr)

    acc_num, acc_seq = await get_account_num_seq(sender_acc_addr)

    async with grpc.aio.insecure_channel('testnet-sentry0.nibiru.network:9910') as channel:
        accounts_rpc = accounts_rpc_grpc.nibiruAccountsRPCStub(channel)
        account_addr = "inj14au322k9munkmx5wrchz9q30juf5wjgz2cfqku"

        subacc = await accounts_rpc.SubaccountsList(accounts_rpc_pb.SubaccountsListRequest(account_address = account_addr))
        for sub in subacc.subaccounts:
            print("Primary subaccount:", sub)
            break

    tx = Transaction(
        privkey=sender_pk,
        account_num=acc_num,
        sequence=acc_seq,
        gas=200000,
        fee=200000 * MIN_GAS_PRICE,
        sync_mode="block",
    )
    tx.add_cosmos_bank_msg_send(
        recipient="inj1qy69k458ppmj45c3vqwcd6wvlcuvk23x0hsz58",
        amount=10000000000000000,
        denom="inj",
    )
    tx.add_exchange_msg_deposit(
        subaccount= sub,
        amount=10000000000000000,
        denom="inj",
    )

    tx_json = tx.get_signed()
    tx_result = await post_tx(tx_json)

    print("Signed Tx:", tx_json)
    print("Sent Tx:", tx_result)

    return len(tx_result)

@pytest.mark.asyncio
async def test_msg_send(msg_send):
    assert msg_send == 64


"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.feegrant.v1beta1.feegrant_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class QueryAllowanceRequest(google.protobuf.message.Message):
    """QueryAllowanceRequest is the request type for the Query/Allowance RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GRANTER_FIELD_NUMBER: builtins.int
    GRANTEE_FIELD_NUMBER: builtins.int
    granter: typing.Text
    """granter is the address of the user granting an allowance of their funds."""

    grantee: typing.Text
    """grantee is the address of the user being granted an allowance of another user's funds."""

    def __init__(self,
        *,
        granter: typing.Text = ...,
        grantee: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["grantee",b"grantee","granter",b"granter"]) -> None: ...
global___QueryAllowanceRequest = QueryAllowanceRequest

class QueryAllowanceResponse(google.protobuf.message.Message):
    """QueryAllowanceResponse is the response type for the Query/Allowance RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ALLOWANCE_FIELD_NUMBER: builtins.int
    @property
    def allowance(self) -> cosmos.feegrant.v1beta1.feegrant_pb2.Grant:
        """allowance is a allowance granted for grantee by granter."""
        pass
    def __init__(self,
        *,
        allowance: typing.Optional[cosmos.feegrant.v1beta1.feegrant_pb2.Grant] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["allowance",b"allowance"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allowance",b"allowance"]) -> None: ...
global___QueryAllowanceResponse = QueryAllowanceResponse

class QueryAllowancesRequest(google.protobuf.message.Message):
    """QueryAllowancesRequest is the request type for the Query/Allowances RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GRANTEE_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    grantee: typing.Text
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an pagination for the request."""
        pass
    def __init__(self,
        *,
        grantee: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["grantee",b"grantee","pagination",b"pagination"]) -> None: ...
global___QueryAllowancesRequest = QueryAllowancesRequest

class QueryAllowancesResponse(google.protobuf.message.Message):
    """QueryAllowancesResponse is the response type for the Query/Allowances RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ALLOWANCES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def allowances(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.feegrant.v1beta1.feegrant_pb2.Grant]:
        """allowances are allowance's granted for grantee by granter."""
        pass
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines an pagination for the response."""
        pass
    def __init__(self,
        *,
        allowances: typing.Optional[typing.Iterable[cosmos.feegrant.v1beta1.feegrant_pb2.Grant]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allowances",b"allowances","pagination",b"pagination"]) -> None: ...
global___QueryAllowancesResponse = QueryAllowancesResponse

class QueryAllowancesByGranterRequest(google.protobuf.message.Message):
    """QueryAllowancesByGranterRequest is the request type for the Query/AllowancesByGranter RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GRANTER_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    granter: typing.Text
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an pagination for the request."""
        pass
    def __init__(self,
        *,
        granter: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["granter",b"granter","pagination",b"pagination"]) -> None: ...
global___QueryAllowancesByGranterRequest = QueryAllowancesByGranterRequest

class QueryAllowancesByGranterResponse(google.protobuf.message.Message):
    """QueryAllowancesByGranterResponse is the response type for the Query/AllowancesByGranter RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ALLOWANCES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def allowances(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.feegrant.v1beta1.feegrant_pb2.Grant]:
        """allowances that have been issued by the granter."""
        pass
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines an pagination for the response."""
        pass
    def __init__(self,
        *,
        allowances: typing.Optional[typing.Iterable[cosmos.feegrant.v1beta1.feegrant_pb2.Grant]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allowances",b"allowances","pagination",b"pagination"]) -> None: ...
global___QueryAllowancesByGranterResponse = QueryAllowancesByGranterResponse



# Copyright 2022 Nibiru Chain
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import logging

from nibiru import Composer, Sdk


async def main() -> None:
    sender = Sdk.authorize(
        "guard cream sadness conduct invite crumble clock pudding hole grit liar hotel maid produce squeeze return argue turtle know drive eight casino maze host"
    )
    receiver = Sdk.authorize()

    res = await sender.tx.add_messages(
        Composer.msg_send(
            from_address=sender.address,
            to_address=receiver.address,
            coins=[Composer.coin(amount=5, denom="unibi")],
            # additional params for gas price can be passed
            # gas_price = 7,
            # gas_multiplier = 1.25,
            # gas_wanted = 50_000,
        ),
        Composer.msg_send(
            from_address=sender.address,
            to_address=receiver.address,
            coins=[Composer.coin(amount=7, denom="unusd")],
        ),
    ).execute()
    logging.info("Result: %s", res)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    asyncio.get_event_loop().run_until_complete(main())



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import epochs.query_pb2
import grpc

class QueryStub:
    """Query defines the gRPC querier service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    EpochInfos: grpc.UnaryUnaryMultiCallable[
        epochs.query_pb2.QueryEpochsInfoRequest,
        epochs.query_pb2.QueryEpochsInfoResponse]
    """EpochInfos provide running epochInfos"""

    CurrentEpoch: grpc.UnaryUnaryMultiCallable[
        epochs.query_pb2.QueryCurrentEpochRequest,
        epochs.query_pb2.QueryCurrentEpochResponse]
    """CurrentEpoch provide current epoch of specified identifier"""


class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service."""
    @abc.abstractmethod
    def EpochInfos(self,
        request: epochs.query_pb2.QueryEpochsInfoRequest,
        context: grpc.ServicerContext,
    ) -> epochs.query_pb2.QueryEpochsInfoResponse:
        """EpochInfos provide running epochInfos"""
        pass

    @abc.abstractmethod
    def CurrentEpoch(self,
        request: epochs.query_pb2.QueryCurrentEpochRequest,
        context: grpc.ServicerContext,
    ) -> epochs.query_pb2.QueryCurrentEpochResponse:
        """CurrentEpoch provide current epoch of specified identifier"""
        pass


def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import lockup.v1.lock_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class GenesisState(google.protobuf.message.Message):
    """GenesisState defines the lockup module's genesis state."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LOCKS_FIELD_NUMBER: builtins.int
    @property
    def locks(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[lockup.v1.lock_pb2.Lock]: ...
    def __init__(self,
        *,
        locks: typing.Optional[typing.Iterable[lockup.v1.lock_pb2.Lock]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["locks",b"locks"]) -> None: ...
global___GenesisState = GenesisState
